#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include "picture_coder.h"
#include "vvl_defs.h"
#include "cpia.h"
#include "mcam.h"
#include "internet.h"

extern int debug_flag;
extern char *frame_dir;
extern BYTE __huge *gStreamBuffer;
extern BYTE __huge *DataBuffer;
extern int coding_quality;


int gstream_to_y_u_v_files(\
	int xsize, int ysize, unsigned char *inbuffer, int inbuffer_size,\
	int compression_flag)
{
int a, c, i, j;
int low_byte, high_byte;
int line_length;
BYTE __huge *ptr;
int line_cnt, col_cnt;
int y_idx, u_idx, v_idx;
int y_matrix[288][352];
int u_matrix[288][176];
int v_matrix[288][176];
int y_val, u_val, v_val;
int y_width, u_width, v_width;
int y_replace, u_replace, v_replace;
FILE *fptr;
char temp[1024];

if(debug_flag)
	{
	fprintf(stderr, "gstream_to_y_u_v_files():\n\
	arg xsize=%d ysize=%d inbuffer=%lu inbuffer_size=%d\n\
	fptr=%lu compression_flag=%d\n",\
	xsize, ysize, (long)inbuffer, inbuffer_size,\
	(long)fptr, compression_flag);
	}

/* cannot process compressed buffer */
//if(inbuffer[COMP_ENABLE] == 1)
//	{
//	fprintf(stderr,\
//	"gstream_to_y_u_v_files(): No wavelet coding when compression is on\n");
//	return 1;
//	}

/* fill the Y U V matrices */

/*
scip the header, point to first line.
format is line length low, line length high, YUYV, EOL (0xfd), 
frame ends with 0xff 0xff 0xff 0xff
*/
ptr = inbuffer + 64;
line_cnt = 0;
while(1)
	{
	/* low byte line length */
	low_byte = *ptr;
	ptr++;

	/* high byte line length */
	high_byte = *ptr;
	ptr++;

	/* get length of this line */
	line_length = low_byte + (256 * high_byte);

	if(debug_flag)
		{
		fprintf(stderr, "detect_change(): line=%d line_length=%d\n",\
		line_cnt, line_length);
		}

	/* test for last line (255 in both low and high position) line length */
	if(line_length == 65535) break; 
	/* index in array */
	y_idx = 0;
	u_idx = 0;
	v_idx = 0;
	/* process the line */
	i = 0;
	for(i = 0; i < line_length / 4; i++)
		{
		/* first y */
		/* to luminance matrix */
		y_matrix[line_cnt][y_idx] = *ptr;
		ptr++;
		y_idx++;
		
		/* u (blue vector) 255 is blue, 127 is no color, 0 is yellowish */
		/* to u matrix */
		u_matrix[line_cnt][u_idx] = *ptr;
		ptr++;
		u_idx++;
		
		/* second y */
		/* to luminance matrix */
		y_matrix[line_cnt][y_idx] = *ptr;
		ptr++;
		y_idx++;

		/* v (red vector) 255 is red, 127 = no color, 0 is greenish */
		/* to v matrix */
		v_matrix[line_cnt][v_idx] = *ptr;
		ptr++;
		v_idx++;
		}/* end for each line */
	ptr++;

	line_cnt++;
	}/* end for all lines */


if(debug_flag)
	{
	fprintf(stderr, "gstream_to_y_u_v_files(): last line detected\n");
	}

/* write the Y .pgm file */
sprintf(temp, "%s/mcam.y.pgm", frame_dir);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fprintf(stderr,\
	"gstream_to_y_u_v_files(): could not open file %s for write\n", temp);
	
	return 0;
	}

/* write .pgm type P5 header (256 steps grey scale) */
fprintf(fptr, "P5\n%i %i\n255\n", xsize, ysize);

/* write the data */
for(line_cnt = 0; line_cnt < ysize; line_cnt++)
	{
	for(col_cnt = 0; col_cnt < y_idx; col_cnt++)
		{
		fputc(y_matrix[line_cnt][col_cnt], fptr);
		}
	}

/* close the file */
fclose(fptr);

/* write the U .pgm file */
sprintf(temp, "%s/mcam.u.pgm", frame_dir);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fprintf(stderr,\
	"gstream_to_y_u_v_files(): could not open file %s for write\n", temp);
	
	return 0;
	}

/* write .pgm type P5 header (256 steps grey scale) */
fprintf(fptr, "P5\n%i %i\n255\n", xsize / 2, ysize);

/* write the data */
for(line_cnt = 0; line_cnt < ysize; line_cnt++)
	{
	for(col_cnt = 0; col_cnt < u_idx; col_cnt++)
		{
		fputc(u_matrix[line_cnt][col_cnt], fptr);
		}
	}

/* close the file */
fclose(fptr);

/* write the V .pgm file */
sprintf(temp, "%s/mcam.v.pgm", frame_dir);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fprintf(stderr,\
	"gstream_to_y_u_v_files(): could not open file %s for write\n", temp);
	
	return 0;
	}

/* write .pgm type P5 header (256 steps grey scale) */
fprintf(fptr, "P5\n%i %i\n255\n", xsize / 2, ysize);

/* write the data */
for(line_cnt = 0; line_cnt < ysize; line_cnt++)
	{
	for(col_cnt = 0; col_cnt < v_idx; col_cnt++)
		{
		fputc(v_matrix[line_cnt][col_cnt], fptr);
		}
	}

/* close the file */
fclose(fptr);

return 1;
}/* end function gstream_to_y_u_v_files */


int y_u_v_files_to_gstream(unsigned char *inbuffer, int *bytes_read)
{
int a, b, c, i;
char temp[1024];
char *ptr;
FILE *fptr;
int y_byte_cnt, u_byte_cnt, v_byte_cnt;
int total_cnt;

if(debug_flag)
	{
	fprintf(stderr,\
	"y_u_v_files_to_gstream(): arg inbuffer=%lu\n",\
	(long)inbuffer);
	}

ptr = inbuffer + 64;

/*
length_y_high
length_y mid
length_y low
length_u_high
length_u mid
length_u low
length_v_high
length_v_mid
length_v_low
*/
ptr += 9;

/* read the Y wavelet file */
sprintf(temp, "%s/mcam.y.wavelet", frame_dir);
fptr = fopen(temp, "r");
if(! fptr)
	{
	fprintf(stderr,\
	"y_u_v_files_to_gstream(): could not open file %s for read\n", temp);
	
	return 0;
	}

y_byte_cnt = 0;
/* read Y wavelet into buffer data area + 3;*/
while(1)
	{
	while(1)
		{
		errno = 0;
		c = getc(fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}

	if(c == EOF) break;

	*ptr = c;	
	ptr++;

	y_byte_cnt++;
	}/* end while all bytes in y.wavelet file */

fclose(fptr);

/* high byte first */
*(inbuffer + 64 + 0) = y_byte_cnt / 65536; 
*(inbuffer + 64 + 1) = y_byte_cnt / 256;
*(inbuffer + 64 + 2) = y_byte_cnt % 256;

/* read the U wavelet file */
sprintf(temp, "%s/mcam.u.wavelet", frame_dir);
fptr = fopen(temp, "r");
if(! fptr)
	{
	fprintf(stderr,\
	"y_u_v_files_to_gstream(): could not open file %s for read\n", temp);
	
	return 0;
	}

u_byte_cnt = 0;
while(1)
	{
	while(1)
		{
		errno = 0;
		c = getc(fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}

	if(c == EOF) break;

	*ptr = c;	
	ptr++;

	u_byte_cnt++;
	}/* end while all bytes in y.wavelet file */

fclose(fptr);

/* high byte first */
*(inbuffer + 64 + 3) = u_byte_cnt / 65536; 
*(inbuffer + 64 + 4) = u_byte_cnt / 256;
*(inbuffer + 64 + 5) = u_byte_cnt % 256;

/* read the V wavelet file */
sprintf(temp, "%s/mcam.v.wavelet", frame_dir);
fptr = fopen(temp, "r");
if(! fptr)
	{
	fprintf(stderr,\
	"y_u_v_files_to_gstream(): could not open file %s for read\n", temp);
	
	return 0;
	}

v_byte_cnt = 0;
while(1)
	{
	while(1)
		{
		errno = 0;
		c = getc(fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}

	if(c == EOF) break;

	*ptr = c;	
	ptr++;

	v_byte_cnt++;
	}/* end while all bytes in y.wavelet file */

fclose(fptr);

/* high byte first */
*(inbuffer + 64 + 6) = v_byte_cnt / 65536; 
*(inbuffer + 64 + 7) = v_byte_cnt / 256;
*(inbuffer + 64 + 8) = v_byte_cnt % 256;

*bytes_read = 9 + y_byte_cnt + u_byte_cnt + v_byte_cnt;

return 1;
}/* end function y_u_v_files_to_gstream */


int make_wavelet_in_buffer(int *bytes_read)
{
int a, c, i;
DWORD actual_bytes;
BYTE stream;
unsigned char *ucptr;
char temp[1024];
FILE *fptr;
FILE *pptr;
int compression_flag;
int size;
int xsize, ysize;
float compression_factor;

if(debug_flag)
	{
	fprintf(stderr, "make_wavelet_in_buffer() arg none\n");
	}

/* get x and y size from frame buffer header */
if (gStreamBuffer[VIDEO_SIZE] == VIDEOSIZE_CIF)
	{
	xsize =\
	(gStreamBuffer[COL_END]-gStreamBuffer[COL_START]) << 3;
	ysize =\
	(gStreamBuffer[ROW_END]-gStreamBuffer[ROW_START]) << 2;
	}
else
	{
	xsize =\
	(gStreamBuffer[COL_END]-gStreamBuffer[COL_START]) << 2;
	ysize =\
	(gStreamBuffer[ROW_END]-gStreamBuffer[ROW_START]) << 1;
	}
if(debug_flag)
	{
	fprintf(stderr, "make_wavelet_in_buffer(): coding xsize=%d ysize=%d\n",\
	xsize, ysize);
	}
	
size = UNKNOWN;
if( (xsize == 352) && (ysize == 288) ) size = BIG;
if( (xsize == 176) && (ysize == 144) ) size = SMALL;
if(size == UNKNOWN)
	{
	fprintf(stderr, "make_wavelet_in_buffer(): wavelet xsize=%d ysize=%d aborting\n",\
	xsize, ysize);
	return 0;
	}

compression_flag = DataBuffer[HEADER_SIZE + 28];

/* this will create in frame_dir:
mcam-wavelet.y,  mcam-wavelet.u,  and mcam-wavelet.v
in .pgm format (P5)
*/
if(! gstream_to_y_u_v_files(\
xsize, ysize, gStreamBuffer, *bytes_read, compression_flag) )
	{
	return 0;
	}

/*
Going to use the 'epic' wavelet codec from:
 Prof. Eero P. Simoncelli
GRASP Laboratory, Rm 335C
3401 Walnut Street
Philadelphia, PA  19104-6228

E-mail: eero@central.cis.upenn.edu
WWW: http://www.cis.upenn.edu/~eero/home.html

THE WAVELET CODING PROGRAM 'epic' IS FOR NON COMMERCIAL USE ONLY,
IT IS FORBIDDEN TO REMOVE THIS NOTICE FROM THE CODE

This requires 3 coding steps, one for Y, one for U and one for V
*/

compression_factor = 100.0 / coding_quality;

sprintf(temp,\
"epic %s/mcam.y.pgm -o %s/mcam.y.wavelet -b %.2f 1> /dev/null 2> /dev/null",\
frame_dir, frame_dir, compression_factor);
pptr = popen(temp, "r");
pclose(pptr);
if(debug_flag)
	{
	fprintf(stderr, "make_wavelet_in_buffer(): wavelet executing=%s\n", temp);
	}
pptr = popen(temp, "r");
pclose(pptr);

sprintf(temp,\
"epic %s/mcam.u.pgm -o %s/mcam.u.wavelet -b %.2f 1> /dev/null 2> /dev/null",\
frame_dir, frame_dir, compression_factor);
pptr = popen(temp, "r");
pclose(pptr);
if(debug_flag)
	{
	fprintf(stderr, "make_wavelet_in_buffer(): wavelet executing=%s\n", temp);
	}
pptr = popen(temp, "r");
pclose(pptr);

sprintf(temp,\
"epic %s/mcam.v.pgm -o %s/mcam.v.wavelet -b %.2f 1> /dev/null 2> /dev/null",\
frame_dir, frame_dir, compression_factor);
pptr = popen(temp, "r");
pclose(pptr);

if(debug_flag)
	{
	fprintf(stderr, "make_wavelet_in_buffer(): wavelet executing=%s\n", temp);
	}
pptr = popen(temp, "r");
pclose(pptr);

/* bitstream to gStreamBuffer + 64 */

/*
This will read in frame_dir 
mcam.y.wavelet,  mcam.u.wavelet, and mcam.v.wavelet
in .pgm format (P5)
and add 2 length bytes at the start of each, and add these 3 parts to
the gstream buffer.

lh, ll, data, lh, ll, data, lh, ll, data 
*/

if(! y_u_v_files_to_gstream(gStreamBuffer, &i) )
	{
	return 0;
	}
	
/* header + data */
*bytes_read = i + 64;
	
if(debug_flag)
	{
	fprintf(stderr,\
	"make_wavelet_in_buffer(): wavelet bytes_read=%d\n",\
	*bytes_read);
	}
	
return 1;
}/* end function make_wavelet_in_buffer */


int convert_wavelet_in_buffer_to_gstream(int *bytes_in_buffer)
{
int c, i, j;
FILE *fptr;
char temp[1024];
FILE *pptr;
char *ptr;
int y_length, u_length, v_length;
int total_byte_cnt;
int width, height, maxval;
int line_length;
BYTE __huge *out_ptr;
int bytes_out;
int state;
int comment_flag;

#define START_STATE		0
#define WIDTH_STATE		1
#define HEIGHT_STATE	2
#define MAXVAL_STATE	3
#define DATA_STATE		4

if(debug_flag)
	{
	fprintf(stderr, "convert_wavelet_in_buffer_to_gstream(): arg none\n");
	}

/*
write gstream buffer to 3 files in frame_dir:
mcam.y.b.wavelet, mcam.u.b.wavelet, mcam.v.b.wavelet
*/

y_length =\
	gStreamBuffer[64 + 0] * 65536 +\
	gStreamBuffer[64 + 1] * 256 +\
	gStreamBuffer[64 + 2];

u_length =\
	gStreamBuffer[64 + 3] * 65536 +\
	gStreamBuffer[64 + 4] * 256 +\
	gStreamBuffer[64 + 5];

v_length =\
	gStreamBuffer[64 + 6] * 65536 +\
	gStreamBuffer[64 + 7] * 256 +\
	gStreamBuffer[64 + 8];

if(debug_flag)
	{
	fprintf(stderr,\
	"convert_wavelet_in_buffer_to_gstream():\n\
	 y_length=%d u_length=%d v_length=%d\n",\
	y_length, u_length, v_length);
	}

/* write mcam.y.b.wavelet */
sprintf(temp, "%s/mcam.y.b.wavelet", frame_dir);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fprintf(stderr, "convert_wavelet_in_buffer_to_gstream():\n\
	could not open file %s for write\n", temp);

	return 0;
	}

for(i = 0; i < y_length; i++)
	{
	while(1)
		{
		errno = 0;
		fputc(gStreamBuffer[64 + 9 + i], fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}
	}
fclose(fptr);

/* write mcam.u.b.wavelet */
sprintf(temp, "%s/mcam.u.b.wavelet", frame_dir);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fprintf(stderr, "convert_wavelet_in_buffer_to_gstream():\n\
	could not open file %s for write\n", temp);

	return 0;
	}

for(i = 0; i < u_length; i++)
	{
	while(1)
		{
		errno = 0;
		fputc(gStreamBuffer[64 + 9 + y_length + i], fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}
	}
fclose(fptr);

/* mcam.v.b.wavelet */
sprintf(temp, "%s/mcam.v.b.wavelet", frame_dir);
fptr = fopen(temp, "w");
if(! fptr)
	{
	fprintf(stderr, "convert_wavelet_in_buffer_to_gstream():\n\
	could not open file %s for write\n", temp);

	return 0;
	}

for(i = 0; i < v_length; i++)
	{
	while(1)
		{
		errno = 0;
		fputc(gStreamBuffer[64 + 9 + y_length + u_length + i], fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}
	}
fclose(fptr);

/*
use unepic on each file, this creates: 
mcam.y.b.pgm, mcam.u.b.pgm, mcam.v.b.pgm
*/

/* transform mcam.y.b.wavelet to mcam.y.b.pgm */
sprintf(temp, "unepic %s/mcam.y.b.wavelet %s/mcam.y.b.pgm",\
frame_dir, frame_dir);
if(debug_flag)
	{
	fprintf(stderr,\
	"convert_wavelet_in_buffer_to_gstream():\n\
	going to execute %s\n", temp);
	}
pptr = popen(temp, "r");
pclose(pptr);

/* transform mcam.u.b.wavelet to mcam.u.b.pgm */
sprintf(temp, "unepic %s/mcam.u.b.wavelet %s/mcam.u.b.pgm",\
frame_dir, frame_dir);
if(debug_flag)
	{
	fprintf(stderr,\
	"convert_wavelet_in_buffer_to_gstream():\n\
	going to execute %s\n", temp);
	}
pptr = popen(temp, "r");
pclose(pptr);

/* transform mcam.v.b.wavelet to mcam.v.b.pgm */
sprintf(temp, "unepic %s/mcam.v.b.wavelet %s/mcam.v.b.pgm",\
frame_dir, frame_dir);
if(debug_flag)
	{
	fprintf(stderr,\
	"convert_wavelet_in_buffer_to_gstream():\n\
	going to execute %s\n", temp);
	}
pptr = popen(temp, "r");
pclose(pptr);

/*
read mcam.y.b.pgm, mcam.u.b.pgm, mcam.y.b.pgm
into the correct places in the gstream buffer, and set the bytes_read.
*/

/* need to return this in *bytes_in_buffer */
total_byte_cnt = 0;

/* read mcam.y.b.pgm */
sprintf(temp, "%s/mcam.y.b.pgm", frame_dir);
fptr = fopen(temp, "r");
if(! fptr)
	{
	fprintf(stderr,\
	"convert_wavelet_in_buffer_to_gstream():\n\
	could not open file %s for read\n", temp);

	return 0;
	}

/*
read the header to get width, height, and maxval
*/

/*
unepic writes like this:
P5
# CREATOR: UNEPIC, Version 1.10
352 288
255
*/

/*
using fscanf here swallows control M after \n, resulting in wrong color,
perhaps a libc bug?
*/

/* parse the header */
comment_flag = 0;
state = START_STATE;
i = 0;
while(1)
	{
	while(1)
		{
		errno = 0;
		c = getc(fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}
	if(c == EOF)
		{
		fclose(fptr);
		fprintf(stderr,\
		"convert_wavelet_in_buffer_to_gstream():\n\
		pgm_to_gstream(): early EOF in header\n");
		return 0;
		}

	/* handle comments */
	if(c == '\n') comment_flag = 0;
	if(c == '#') comment_flag = 1;
	if(comment_flag) continue;

	temp[i] = c;

	/* detect end field */
	if( (c == '\n') || (c == '\t') || (c == '\r') || (c == ' ') )
		{
		/* string termination line */
		temp[i] = 0;

		/* if any valid data in this field use it */
		if(strlen(temp) != 0)
			{
			switch(state)
				{
				case START_STATE:
					/* scip the P5 */
					state = WIDTH_STATE;
					break;
				case WIDTH_STATE:
					width = atoi(temp);
					state = HEIGHT_STATE;
					break;
				case HEIGHT_STATE:
					height = atoi(temp);
					state = MAXVAL_STATE;
					break;
				case MAXVAL_STATE:
					maxval = atoi(temp);
					state = DATA_STATE;
					break;
				case DATA_STATE:
					break;
				}/* end switch */
			}/* end if valid data in field */

		/* point to start line */
		i = 0;
		}/* end if white space */
	else i++;

	if(state == DATA_STATE) break;	
	}/* end for all chars in header */

if(debug_flag)
	{
	fprintf(stderr,\
	"convert_wavelet_in_buffer_to_gstream():\n\
	pgm_to_gstream(): Y width=%d height=%d maxval=%d\n",\
	width, height, maxval); 
	}

/* gstream line length is 352 * 2 + EOL or 176 * 2 + EOL */
line_length = (width * 2) + 1; 

/*
YUYV
*/

/* start at beginning of data area, after the 64 byte header */
out_ptr = gStreamBuffer + 64;

/*
Create a frame in gstream buffer
writing Y only, the line length, EOL, and EOF markers.
*/
for (i = 0; i < height; i++)
	{
	/* gstream low byte line_length */
	*out_ptr = line_length % 256;
	out_ptr++;	

	/* gstream high byte line_lenght */
	*out_ptr = line_length / 256;
	out_ptr++;
		
	total_byte_cnt += 2;

	/* write Y, (U), Y, (V) */
	for(j = 0; j < width; j++)
		{
		while(1)
			{
			errno = 0;
			c = getc(fptr);
			if(errno == EAGAIN) continue;
			if(errno == EINTR) continue;
			break;
			}

		if(c == EOF) break;

		/* write y */
		*out_ptr = c;
		out_ptr++;
		total_byte_cnt++;

		/* point to next Y (scip U or V) */
		out_ptr ++;
		total_byte_cnt++;

		}/* end for j horizontal */

	/* write gstream EOL marker */
	*out_ptr = 0xfd;
	out_ptr++;
	total_byte_cnt ++;

	}/* end for i vertical */

/* close the input file */
fclose(fptr);

/* write gstream EOF markers */
*out_ptr = 0xff;
out_ptr++;
*out_ptr = 0xff;
out_ptr++;
*out_ptr = 0xff;
out_ptr++;
*out_ptr = 0xff;
total_byte_cnt += 4;

/* read mcam.u.b.pgm */
sprintf(temp, "%s/mcam.u.b.pgm", frame_dir);
fptr = fopen(temp, "r");
if(! fptr)
	{
	fprintf(stderr,\
	"convert_wavelet_in_buffer_to_gstream():\n\
	could not open file %s for read\n", temp);

	return 0;
	}

/*
read the header to get width, height, and maxval
*/

/*
unepic writes like this:
P5
# CREATOR: UNEPIC, Version 1.10
352 288
255
*/

/*
using fscanf here swallows control M after \n, resulting in wrong color,
perhaps a libc bug?
*/

/* parse the header */
comment_flag = 0;
state = START_STATE;
i = 0;
while(1)
	{
	while(1)
		{
		errno = 0;
		c = getc(fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}
	if(c == EOF)
		{
		fclose(fptr);
		fprintf(stderr,\
		"convert_wavelet_in_buffer_to_gstream():\n\
		pgm_to_gstream(): early EOF in header\n");
		return 0;
		}

	/* handle comments */
	if(c == '\n') comment_flag = 0;
	if(c == '#') comment_flag = 1;
	if(comment_flag) continue;

	temp[i] = c;

	/* detect end field */
	if( (c == '\n') || (c == '\t') || (c == '\r') || (c == ' ') )
		{
		/* string termination line */
		temp[i] = 0;

		/* if any valid data in this field use it */
		if(strlen(temp) != 0)
			{
			switch(state)
				{
				case START_STATE:
					/* scip the P5 */
					state = WIDTH_STATE;
					break;
				case WIDTH_STATE:
					width = atoi(temp);
					state = HEIGHT_STATE;
					break;
				case HEIGHT_STATE:
					height = atoi(temp);
					state = MAXVAL_STATE;
					break;
				case MAXVAL_STATE:
					maxval = atoi(temp);
					state = DATA_STATE;
					break;
				case DATA_STATE:
					break;
				}/* end switch */
			}/* end if valid data in field */

		/* point to start line */
		i = 0;
		}/* end if white space */
	else i++;

	if(state == DATA_STATE) break;	
	}/* end for all chars in header */

if(debug_flag)
	{
	fprintf(stderr,\
	"convert_wavelet_in_buffer_to_gstream():\n\
	pgm_to_gstream(): U width=%d height=%d maxval=%d\n",\
	width, height, maxval); 
	}

/*
writing U.
*/
out_ptr = gStreamBuffer + 64;
for (i = 0; i < height; i++)
	{
	/* scip the line length bytes */
	out_ptr += 2;

	/* write (Y), U, (Y), (V), (Y)*/
	for(j = 0; j < width; j++)
		{
		while(1)
			{
			errno = 0;
			c = getc(fptr);
			if(errno == EAGAIN) continue;
			if(errno == EINTR) continue;
			break;
			}

		if(c == EOF) break;

		/* write u */
		*(out_ptr + 1) = c;

		/* point to next u */
		out_ptr += 4;

		}/* end for j horizontal */

	/* scip EOL marker */
	out_ptr += 1;
	}/* end for i vertical */

/* close the input file */
fclose(fptr);

/* read mcam.v.b.pgm */
sprintf(temp, "%s/mcam.v.b.pgm", frame_dir);
fptr = fopen(temp, "r");
if(! fptr)
	{
	fprintf(stderr,\
	"convert_wavelet_in_buffer_to_gstream():\n\
	could not open file %s for read\n", temp);

	return 0;
	}

/*
read the header to get width, height, and maxval
*/

/*
unepic writes like this:
P5
# CREATOR: UNEPIC, Version 1.10
352 288
255
*/

/*
using fscanf here swallows control M after \n, resulting in wrong color,
perhaps a libc bug?
*/

/* parse the header */
comment_flag = 0;
state = START_STATE;
i = 0;
while(1)
	{
	while(1)
		{
		errno = 0;
		c = getc(fptr);
		if(errno == EAGAIN) continue;
		if(errno == EINTR) continue;
		break;
		}
	if(c == EOF)
		{
		fclose(fptr);
		fprintf(stderr,\
		"convert_wavelet_in_buffer_to_gstream():\n\
		pgm_to_gstream(): early EOF in header\n");
		return 0;
		}

	/* handle comments */
	if(c == '\n') comment_flag = 0;
	if(c == '#') comment_flag = 1;
	if(comment_flag) continue;

	temp[i] = c;

	/* detect end field */
	if( (c == '\n') || (c == '\t') || (c == '\r') || (c == ' ') )
		{
		/* string termination line */
		temp[i] = 0;

		/* if any valid data in this field use it */
		if(strlen(temp) != 0)
			{
			switch(state)
				{
				case START_STATE:
					/* scip the P5 */
					state = WIDTH_STATE;
					break;
				case WIDTH_STATE:
					width = atoi(temp);
					state = HEIGHT_STATE;
					break;
				case HEIGHT_STATE:
					height = atoi(temp);
					state = MAXVAL_STATE;
					break;
				case MAXVAL_STATE:
					maxval = atoi(temp);
					state = DATA_STATE;
					break;
				case DATA_STATE:
					break;
				}/* end switch */
			}/* end if valid data in field */

		/* point to start line */
		i = 0;
		}/* end if white space */
	else i++;

	if(state == DATA_STATE) break;	
	}/* end for all chars in header */

if(debug_flag)
	{
	fprintf(stderr,\
	"convert_wavelet_in_buffer_to_gstream():\n\
	pgm_to_gstream(): V width=%d height=%d maxval=%d\n",\
	width, height, maxval); 
	}

/*
writing V.
*/
out_ptr = gStreamBuffer + 64;
for (i = 0; i < height; i++)
	{
	/* scip the line length bytes */
	out_ptr += 2;

	/* write (Y), (U), (Y), V */
	for(j = 0; j < width; j++)
		{
		while(1)
			{
			errno = 0;
			c = getc(fptr);
			if(errno == EAGAIN) continue;
			if(errno == EINTR) continue;
			break;
			}

		if(c == EOF) break;

		/* write v */
		*(out_ptr + 3) = c;

		/* point to next v */
		out_ptr += 4;

		}/* end for j horizontal */

	/* scip EOL marker */
	out_ptr += 1;
	}/* end for i vertical */

/* close the input file */
fclose(fptr);

if(debug_flag)
	{
	fprintf(stderr,\
	"pgm_to_gstream(): total_byte_cnt=%d\n", total_byte_cnt);
	}

*bytes_in_buffer = total_byte_cnt;

return 1;
}/* end function convert_wavelet_in_buffer_to_gstream */


